Package: holiglm
Type: Package
Title: Holistic Generalized Linear Models
Version: 1.0.1
Authors@R: c(
    person("Benjamin", "Schwendinger", role = c("aut", "cre"), email = "benjaminschwe@gmail.com"),
    person("Florian", "Schwendinger", role = "aut"),
    person("Laura", "Vana", role = "aut")
  )
Description: Holistic generalized linear models (HGLMs) extend generalized linear models (GLMs) by enabling the possibility to add further constraints to the model. The 'holiglm' package simplifies estimating HGLMs using convex optimization. Additional information about the package can be found in the reference manual, the 'README' and the accompanying paper <doi:10.18637/jss.v108.i07>.
Depends: R (>= 3.5.0), ROI.plugin.ecos
Imports: slam, checkmate, MASS, SuppDists, ROI (>= 0.3-0)
Encoding: UTF-8
Suggests: ROI.plugin.scs, ROI.plugin.clarabel, tinytest (>= 1.0.0),
        parallel, knitr, rmarkdown, bookdown, DiagrammeR
License: GPL-3
VignetteBuilder: knitr
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2024-12-20 08:54:13 UTC; ben
Author: Benjamin Schwendinger [aut, cre],
  Florian Schwendinger [aut],
  Laura Vana [aut]
Maintainer: Benjamin Schwendinger <benjaminschwe@gmail.com>
Repository: CRAN
Date/Publication: 2024-12-20 09:20:02 UTC
Built: R 4.6.0; ; 2026-04-27 17:35:45 UTC; unix
