# Generated by roxygen2: do not edit by hand

S3method(active_coefficients,hglm)
S3method(active_coefficients,hglm.fit)
S3method(as.OP,hglm_model)
S3method(c,hglmc)
S3method(coef,hglm)
S3method(coef,hglm.fit)
S3method(predict,hglm)
S3method(print,hglm_seq)
S3method(print,hglmc)
S3method(print,lohglmc)
S3method(print,summary.hglm)
S3method(scale_constraint_matrix,matrix)
S3method(scale_constraint_matrix,simple_triplet_matrix)
S3method(solution,hglm)
S3method(solution,hglm.fit)
S3method(summary,hglm)
S3method(summary,hglm.fit)
export(acoef)
export(active_coefficients)
export(agg_binomial)
export(as.OP)
export(cov_matrix)
export(group_equal)
export(group_inout)
export(group_sparsity)
export(hglm)
export(hglm_fit)
export(hglm_model)
export(hglm_seq)
export(holiglm)
export(include)
export(is.hglmc)
export(k_max)
export(linear)
export(lower)
export(pairwise_sign_coherence)
export(rhglm)
export(rho_max)
export(scale_constraint_matrix)
export(sign_coherence)
export(solution)
export(update_objective)
export(upper)
import(ROI)
import(ROI.plugin.ecos)
import(checkmate)
import(slam)
import(stats)
importFrom(ROI,as.OP)
importFrom(ROI,solution)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,str)
importFrom(utils,tail)
